//------------------------------------------------------------------------------
//                      Copyright Tech-Know-Ware Ltd, 2001
//------------------------------------------------------------------------------


// File name:   umf-lib-test.c
//------------------------------------------------------------------------------
// Description: This file has been produced by Tech-Know-Ware to test and 
//				demonstrate the use of the UMF Library.
//
// Notice:      This file is for evaluation purposes only.  It is presented
//              'as is' and comes with no warrenty of any kind.  
//
//              It should be noted that no attempt has been made to make
//              this code 'production grade'. 
//
//              Tech-Know-Ware Ltd shall not be liable for any damages suffered 
//              by anyone or anything due to the use of this file however they 
//				may be caused.
//
//              Copyright Tech-Know-Ware Ltd, 2001, 2002.
//
//              End of Notice.
//
// Module Tag:  umf - Unified Message Format
// Version:     V.95.5
//
// Entry Points: f_umf_cpp_test
//
// Changes:
// P. Cordell	2001-06-04	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6


#include <stdlib.h>
#include <string.h>

#include "umf-lib.hpp"

extern "C" struct sumf_tag_info mumf_umf_test[];



extern "C" int f_umf_cpp_test( char *ap_in_file, char *ap_out_file )
{
    FILE *lh_fin;
	int l_i;
	int l_c;
	c_umf l_umf;


    // Use mode "rb" so that can read UTF-8
    if( (lh_fin = fopen( ap_in_file, "rb" )) != NULL )
        {
        FILE *lh_debug_file = NULL;
        int l_value;
		char lc_data[256];
		char lc_message[1024];
		int l_message_length;

        if( ap_out_file != NULL )
			{
			char lc_cpp_out_file[256] = "cpp-";
			strcat( lc_cpp_out_file, ap_out_file );

            lh_debug_file = fopen( lc_cpp_out_file, "wt" );
			}
        else
            lh_debug_file = NULL;

        if( lh_debug_file == NULL )
            lh_debug_file = stdout;

		// Read in test message
		for( l_i=0; (l_c = fgetc( lh_fin )) != EOF; l_i++ )
			lc_message[l_i] = l_c;


		if( (l_message_length = l_umf.f_is_message_complete( lc_message, l_i )) != -1 )
			fprintf( lh_debug_file, "Message is complete, length: %d\n\n", l_message_length );
		else
			fprintf( lh_debug_file, "Message is incomeplete\n\n" );

		l_umf.f_set_tag_info_root( mumf_umf_test );

        if( l_umf.f_parse_message( lc_message, l_i ) )
			fprintf( lh_debug_file, "Message parsed OK\n\n" );


        fprintf( lh_debug_file, "Results of tree parse operation\n"
                                "-------------------------------\n" );

        l_umf.f_display_parse_info( lh_debug_file );


        fprintf( lh_debug_file, "\n\n" );


        fprintf( lh_debug_file, "Results of extracting parameters\n"
                                "--------------------------------\n" );

        char lc_host_name[128];

		// Set location to first untagged struct
		sumf_location l_input_location;
		l_umf.f_set_location( &l_input_location, "/", 0 );


		// Read first untagged parameter
        if( l_umf.f_read_ascii( "0/0", 0, lc_host_name, sizeof( lc_host_name ) ) )
            fprintf( lh_debug_file, "Untagged host-name is: %s\n", lc_host_name );
        else
            fprintf( lh_debug_file, "Untagged host-name is: NOT FOUND\n" );


		// Read second untagged parameter
        if( l_umf.f_read_integer( "0/1", 0, &l_value ) )
            fprintf( lh_debug_file, "Untagged port parameter: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Untagged port parameter: NOT FOUND\n" );


		// Move back to the root of the message tree
		l_umf.f_restore_location( l_input_location );


		// Read the value of the union tagged by T
		l_umf.f_read_union( "1T", 0, lc_data, sizeof(lc_data) );
        fprintf( lh_debug_file, "Selected UNION in 'T' is: %s\n", lc_data );


		// Enumerate the value of the union tagged by T
		sumf_tag_translation lc_tag_list[] = {
			"TRj", 0,
			"TR", 1,
			"T", 2,
			"", 3,
			NULL, 0 };

		fprintf( lh_debug_file, "Selected UNION in 'T' is: %d\n",
						l_umf.f_read_enumerated_union( "1T", 0, lc_tag_list ) );


        char lc_line_name[128];

		// Move to the struct tagged by T and then TR
        sumf_location l_old_location;
		l_umf.f_set_location( &l_old_location, "T/TR", 0 );

		// Read the first untagged integer
        if( l_umf.f_read_integer( "0", 0, &l_value ) )
            fprintf( lh_debug_file, "Transaction Id: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Transaction Id: NOT FOUND\n" );


		// Read the first untagged integer in the A struct
        if( l_umf.f_read_integer( "A/0", 0, &l_value ) )
            fprintf( lh_debug_file, "Context Id: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Context Id: NOT FOUND\n" );


		// Read the value specified in the untagged union in A  cmd
        l_umf.f_read_union( "A/cmd/0", 0, lc_data, sizeof(lc_data) );

        fprintf( lh_debug_file, "Selected 'cmd' is: %s\n", lc_data );


		// Read the untagged ascii string in A cmd
        if( l_umf.f_read_ascii( "A/cmd/1", 0,  lc_line_name, sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "Line name is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "Line name is: NOT FOUND\n" );


		// Move from the root of the message tree to the second instance of cmd in T/TR/A
        l_umf.f_set_location( "/T/TR/A/cmd", 1 ); 
                                                   

		// Read the value of the first untagged union
        l_umf.f_read_union( "0", 0, lc_data, sizeof(lc_data) );
        fprintf( lh_debug_file, "Second Selected 'cmd' is: %s\n", lc_data );


		// Read the ascii string of the second untagged value
        if( l_umf.f_read_ascii( "1", 0, lc_line_name, sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "2nd Line name is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "2nd Line name is: NOT FOUND\n" );


		// Return to the previous location in the message tree
		l_umf.f_restore_location( l_old_location );


        if( l_umf.f_read_union( "T", 1, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' is: NOT FOUND\n" );



        if( l_umf.f_read_union( "T", 2, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' is: NOT FOUND\n" );



		l_umf.f_set_location( "T", 2 );


        if( l_umf.f_read_union( "Test/0", 0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "First untagged union of Test is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "First untagged union of Test is: NOT FOUND\n" );

		
        if( l_umf.f_read_union( "Test/2volume", 0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Volume is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Volume is: NOT FOUND\n" );

        if( l_umf.f_read_union( "Test/2addr", 0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Address type is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Address type is: NOT FOUND\n" );

		
		// Read the IPv4 address value at Test / addr / IPv4
		unsigned char lc_addr[4];
        if( l_umf.f_read_ipv4( "Test/2addr/IPv4", 0, lc_addr ) )
			fprintf( lh_debug_file, "Address is: %d.%d.%d.%d\n", 
													lc_addr[0], lc_addr[1], lc_addr[2], lc_addr[3] );
		else
			fprintf( lh_debug_file, "Address is: NOT FOUND\n" );


		char *lp_ascii;


		// Read the ascii string at Test / dyn and place it in a dynamically allocated buffer
		// -big enough to take the value
        if( l_umf.f_read_ascii_dynamic( "Test/2dyn", 0, &lp_ascii ) )
			fprintf( lh_debug_file, "Dynamic ASCII is: %s\n", lp_ascii );
		else
			fprintf( lh_debug_file, "Dynamic ASCII is: NOT FOUND\n" );

		free( lp_ascii );


		char lc_small_data[7];

		// Try to read the ascii value at Test / dyn into the buffer.  This should FAIL
		// because the lc_data string is too small
        if( l_umf.f_read_ascii( "Test/2dyn", 0, lc_small_data, sizeof(lc_small_data) ) )
			fprintf( lh_debug_file, "Small ASCII is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Small ASCII is: TOO SMALL\n" );


		tumf_unicode lc_unicode[10];

		// Read in the Unicode value
        if( l_umf.f_read_unicode( "Test/2uni", 0, lc_unicode, sizeof(lc_unicode)/sizeof(tumf_unicode) ) )
			fprintf( lh_debug_file, "Unicode is: %c %c %c %c %c\n", 
								lc_unicode[0], lc_unicode[1], lc_unicode[2], lc_unicode[3], lc_unicode[4] );
		else
			fprintf( lh_debug_file, "Unicode is: NOT FOUND\n" );


		char lc_embedded[1024];
		int l_embedded_length;

		// Read in the embedded value at Test / embed.  The length of the embedded string
		// -is placed in l_embedded_length
        if( l_umf.f_read_embedded( "Test/2embed", 0, lc_embedded, sizeof(lc_embedded),
										&l_embedded_length ) )
			{
			lc_embedded[l_embedded_length] = '\0';
			fprintf( lh_debug_file, "Embedded is: %s\n", 
								lc_embedded );
			}
		else
			fprintf( lh_debug_file, "Embedded is: NOT FOUND\n" );


		// Memory will be released by the destructor

        if( lh_debug_file != stdout )
            fclose( lh_debug_file );

        fclose( lh_fin );
        }


	return( 0 );
}


//------------------------------------------------------------------------------
//                 Copyright Tech-Know-Ware Ltd, 2001, 2002.
//------------------------------------------------------------------------------
