//------------------------------------------------------------------------------
//                      Copyright Tech-Know-Ware Ltd, 2001
//------------------------------------------------------------------------------


// File name:   umf-lib-test.c
//------------------------------------------------------------------------------
// Description: This file has been produced by Tech-Know-Ware to test and 
//				demonstrate the use of the UMF Library.
//
// Notice:      This file is for evaluation purposes only.  It is presented
//              'as is' and comes with no warrenty of any kind.  
//
//              It should be noted that no attempt has been made to make
//              this code 'production grade'.
//
//              Tech-Know-Ware Ltd shall not be liable for any damages suffered 
//              by anyone or anything due to the use of this file however they 
//				may be caused.
//
//              Copyright Tech-Know-Ware Ltd, 2001, 2002.
//
//              End of Notice.
//
// Module Tag:  umf - Unified Message Format
// Version:     V.95.5
//
// Entry Points: main
//
// Changes:
// P. Cordell	2001-06-04	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6


#include <stdlib.h>

#include "umf-lib.h"

extern struct sumf_tag_info mumf_umf_test[];


int f_umf_cpp_test( char *ap_in_file, char *ap_out_file );


void f_umf_c_test( char *ap_in_file, char *ap_out_file )
{
    FILE *lh_fin;
	int l_i;
	int l_c;
    sumf_parse_ctrl l_parse_ctrl;

    // Use mode "rb" so that can read UTF-8
    if( (lh_fin = fopen( ap_in_file, "rb" )) != NULL )
        {
        FILE *lh_debug_file = NULL;
        int l_value;
		unsigned char lc_data[256];
		char lc_message[1024];
		int l_message_length;

        if( ap_out_file != NULL )
            lh_debug_file = fopen( ap_out_file, "wt" );
        else
            lh_debug_file = NULL;

        if( lh_debug_file == NULL )
            lh_debug_file = stdout;

		// Read in test message
		for( l_i=0; (l_c = fgetc( lh_fin )) != EOF; l_i++ )
			lc_message[l_i] = l_c;


		if( (l_message_length = fumf_is_message_complete( lc_message, l_i )) != -1 )
			fprintf( lh_debug_file, "Message is complete, length: %d\n\n", l_message_length );
		else
			fprintf( lh_debug_file, "Message is incomeplete\n\n" );

		fumf_set_tag_info_root( &l_parse_ctrl, mumf_umf_test );

        if( fumf_parse_message( &l_parse_ctrl, lc_message, l_i ) )
			fprintf( lh_debug_file, "Message parsed OK\n\n" );


        fprintf( lh_debug_file, "Results of tree parse operation\n"
                                "-------------------------------\n" );

        fumf_display_parse_info( lh_debug_file, &l_parse_ctrl );


        fprintf( lh_debug_file, "\n\n" );


        fprintf( lh_debug_file, "Results of extracting parameters\n"
                                "--------------------------------\n" );

        {
        char lc_host_name[128];

		// Set location to first untagged struct
		fumf_set_location( &l_parse_ctrl, "/", 0, NULL );


		// Read first untagged parameter
        if( fumf_read_ascii( &l_parse_ctrl,
								"0/0",
								0, 
								lc_host_name,
								sizeof( lc_host_name ) ) )
            fprintf( lh_debug_file, "Untagged host-name is: %s\n", lc_host_name );
        else
            fprintf( lh_debug_file, "Untagged host-name is: NOT FOUND\n" );


		// Read second untagged parameter
        if( fumf_read_integer( &l_parse_ctrl,
								"0/1",
								0, 
								&l_value ) )
            fprintf( lh_debug_file, "Untagged port parameter: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Untagged port parameter: NOT FOUND\n" );


		// Read second instance of second untagged parameter
        if( fumf_read_integer( &l_parse_ctrl,
								"0/1",
								1, 
								&l_value ) )
            fprintf( lh_debug_file, "Second untagged port parameter: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Second untagged port parameter: NOT FOUND\n" );
        }


		// Read the value of the union tagged by T
		fumf_read_union( &l_parse_ctrl,  
							"T",
							0, lc_data, sizeof(lc_data) );
        fprintf( lh_debug_file, "Selected UNION in 'T' is: %s\n", lc_data );

		{
		// Enumerate the value of the union tagged by T
		sumf_tag_translation lc_tag_list[] = {
			"TRj", 0,
			"TR", 1,
			"T", 2,
			"", 3,
			NULL, 0 };

		fprintf( lh_debug_file, "Selected UNION in 'T' is: %d\n",
						fumf_read_enumerated_union( &l_parse_ctrl, 
                                                    "T",
                                                    0, 
													lc_tag_list ) );
		}


        {
        char lc_line_name[128];

		// Move to the struct tagged by T and then TR
        sumf_location l_old_location;

		fumf_set_location( &l_parse_ctrl, "T/TR", 0, &l_old_location );

		// Read the first untagged integer
        if( fumf_read_integer( &l_parse_ctrl,
								"0",
								0, 
								&l_value ) )
            fprintf( lh_debug_file, "Transaction Id: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Transaction Id: NOT FOUND\n" );


		// Read the first untagged integer in the A struct
        if( fumf_read_integer( &l_parse_ctrl,
								"A/0",
								0, 
								&l_value ) )
            fprintf( lh_debug_file, "Context Id: %d\n", l_value );
        else
            fprintf( lh_debug_file, "Context Id: NOT FOUND\n" );


		// Read the value specified in the untagged union in A  cmd
        fumf_read_union( &l_parse_ctrl,  
                            "A/cmd/0",
                            0, lc_data, sizeof(lc_data) );

        fprintf( lh_debug_file, "Selected 'cmd' is: %s\n", lc_data );


		// Read the untagged ascii string in A cmd
        if( fumf_read_ascii( &l_parse_ctrl,
								"A/cmd/1",
								0, 
								lc_line_name,
								sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "Line name is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "Line name is: NOT FOUND\n" );


		// Move from the root of the message tree to the second instance of cmd in T/TR/A
        fumf_set_location( &l_parse_ctrl, "A/cmd", 1, NULL ); 
                                                   

		// Read the value of the first untagged union
        fumf_read_union( &l_parse_ctrl, 
                            "0",
                            0, lc_data, sizeof(lc_data) );
        fprintf( lh_debug_file, "Second Selected 'cmd' is: %s\n", lc_data );


		// Read the ascii string of the second untagged value
        if( fumf_read_ascii( &l_parse_ctrl,
								"1",
								0, 
								lc_line_name,
								sizeof( lc_line_name ) ) )
            fprintf( lh_debug_file, "2nd Line name is: %s\n", lc_line_name );
        else
            fprintf( lh_debug_file, "2nd Line name is: NOT FOUND\n" );


		// Return to the previous location in the message tree
		fumf_restore_location( &l_parse_ctrl, l_old_location );
        }


        if( fumf_read_union( &l_parse_ctrl, 
                            "T",
                            1, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "2nd Selected UNION in 'T' is: NOT FOUND\n" );



        if( fumf_read_union( &l_parse_ctrl, 
                            "T",
                            2, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "3rd Selected UNION in 'T' is: NOT FOUND\n" );



		fumf_set_location( &l_parse_ctrl, "T", 2, NULL );


        if( fumf_read_union( &l_parse_ctrl, 
                            "Test/0",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "First untagged union of Test is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "First untagged union of Test is: NOT FOUND\n" );

		
		{
		// Enumerate the value of the union tagged by T
		sumf_tag_translation lc_tag_list[] = {
			"", 0,
			"*", 1,
			NULL, 0 };

		fprintf( lh_debug_file, "Selected UNION in 'Test/0' is: %d\n",
						fumf_read_enumerated_union( &l_parse_ctrl, 
                                                    "Test/0",
                                                    0, 
													lc_tag_list ) );
		}

		
        if( fumf_read_union( &l_parse_ctrl, 
                            "Test/2volume",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Volume is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Volume is: NOT FOUND\n" );

		
        if( fumf_read_union( &l_parse_ctrl, 
                            "Test/2addr",
                            0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Address type is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Address type is: NOT FOUND\n" );


		{
		// Read the IPv4 address value at Test / addr / IPv4
		unsigned char lc_addr[4];
        if( fumf_read_ipv4( &l_parse_ctrl, 
                            "Test/2addr/IPv4",
                            0, lc_addr ) )
			fprintf( lh_debug_file, "Address is: %d.%d.%d.%d\n", 
													lc_addr[0], lc_addr[1], lc_addr[2], lc_addr[3] );
		else
			fprintf( lh_debug_file, "Address is: NOT FOUND\n" );
		}


		{
		char *lp_ascii;


		// Read the ascii string at Test / dyn and place it in a dynamically allocated buffer
		// -big enough to take the value
        if( fumf_read_ascii_dynamic( &l_parse_ctrl, 
										"Test/2dyn",
										0, &lp_ascii ) )
			{
			fprintf( lh_debug_file, "Dynamic ASCII is: %s\n", lp_ascii );
			free( lp_ascii );
			}
		else
			fprintf( lh_debug_file, "Dynamic ASCII is: NOT FOUND\n" );
		}


		{
		char lc_data[7];

		// Try to read the ascii value at Test / dyn into the buffer.  This should FAIL
		// because the lc_data string is too small
        if( fumf_read_ascii( &l_parse_ctrl, 
										"Test/2dyn",
										0, lc_data, sizeof(lc_data) ) )
			fprintf( lh_debug_file, "Small ASCII is: %s\n", lc_data );
		else
			fprintf( lh_debug_file, "Small ASCII is: TOO SMALL\n" );
		}


		{
		tumf_unicode lc_unicode[10];

		// Read in the Unicode value
        if( fumf_read_unicode( &l_parse_ctrl, 
										"Test/2uni",
										0, lc_unicode, sizeof(lc_unicode)/sizeof(tumf_unicode) ) )
			fprintf( lh_debug_file, "Unicode is: %c %c %c %c %c\n", 
								lc_unicode[0], lc_unicode[1], lc_unicode[2], lc_unicode[3], lc_unicode[4] );
		else
			fprintf( lh_debug_file, "Unicode is: NOT FOUND\n" );
		}


		{
		char lc_embedded[1024];
		int l_embedded_length;

		// Read in the embedded value at Test / embed.  The length of the embedded string
		// -is placed in l_embedded_length
        if( fumf_read_embedded( &l_parse_ctrl, 
										"Test/2embed",
										0, lc_embedded, sizeof(lc_embedded),
										&l_embedded_length ) )
			{
			lc_embedded[l_embedded_length] = '\0';
			fprintf( lh_debug_file, "Embedded is: %s\n", 
								lc_embedded );
			}
		else
			fprintf( lh_debug_file, "Embedded is: NOT FOUND\n" );
		}


		{
		char lc_data[100];
		int l_data_length;
		char lc_reencoded_data[100];

		if( fumf_read_byte_array( &l_parse_ctrl,
							"Test/2binary",
							0, 
							lc_data,
							sizeof( lc_data ) - 1,
							&l_data_length ) )
			{
			lc_data[l_data_length] = '\0';
			fprintf( lh_debug_file, "Binary is: %s\n", 
								lc_data );

			fumf_enc_generate_encoded_byte_array( &l_parse_ctrl, 
													lc_reencoded_data, sizeof( lc_reencoded_data ), 
													lc_data, l_data_length );
			fprintf( lh_debug_file, "Re-encoded Binary is: %s\n", 
								lc_reencoded_data );
			}
		else
			fprintf( lh_debug_file, "Binary is: NOT FOUND\n" );
		}


		{
		tumf_date l_date;
		tumf_time l_time;

		if( fumf_read_date( &l_parse_ctrl,
							"Test/2date",
							0, 
							&l_date ) )
			{
			fprintf( lh_debug_file, "Date is: Day:%d Month:%d Year:%d\n", 
								l_date.day, l_date.month, l_date.year );
			}
		else
			fprintf( lh_debug_file, "Date is: NOT FOUND\n" );

		if( fumf_read_time( &l_parse_ctrl,
							"Test/2time",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND\n" );

		if( fumf_read_time( &l_parse_ctrl,
							"Test/2time2",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time2 is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND\n" );

		if( fumf_read_time( &l_parse_ctrl,
							"Test/2time3",
							0, 
							&l_time ) )
			{
			fprintf( lh_debug_file, "Time3 is: %d %d %d\n", 
								l_time.hour, l_time.min, l_time.sec );
			}
		else
			fprintf( lh_debug_file, "Time is: NOT FOUND\n" );
		}


		// Release the resources associated with the message parsing
        fumf_release( &l_parse_ctrl );


        if( lh_debug_file != stdout )
            fclose( lh_debug_file );

        fclose( lh_fin );
        }
}


int main( int a_argc, char **app_argv )
{
    char *lp_input_file;
    char *lp_output_file;


    if( a_argc >= 2 )
        lp_input_file = app_argv[1];
    else
        lp_input_file = "test-message.txt";

    if( a_argc >= 3 )
        {
        if( app_argv[2][0] != '-' )
            lp_output_file = app_argv[2];
        else
            lp_output_file = NULL;
        }
    else
        lp_output_file = "umf-lib-output.txt";

	f_umf_c_test( lp_input_file, lp_output_file );

	f_umf_cpp_test( lp_input_file, lp_output_file );


	return( 0 );
}


//------------------------------------------------------------------------------
//                 Copyright Tech-Know-Ware Ltd, 2001, 2002.
//------------------------------------------------------------------------------
