//------------------------------------------------------------------------------
//                      Copyright Tech-Know-Ware Ltd, 2001
//------------------------------------------------------------------------------


#ifndef _UMF_LIB_H
#define _UMF_LIB_H


// File name:   umf-lib.h
//------------------------------------------------------------------------------
// Description: This file contains library functions for reading UMF encoded
//				messages.
//
// Notice:      WARNING: This is an early release of this code, and as such may
//				contain a number of bugs.  If you find a bug, please report it
//				to:
//
//					umf-lib-bugs@tech-know-ware.com
//
//				This file is presented 'as is' and comes with no warrenty of 
//				any kind.  
//
//              This file, or any derivative of it, may be used for any 
//				purpose, including commercial purposes.
//
//              Users are permitted to modify the contents of the file, as 
//				long as this notice and the copyright notice remain.
//
//              Users are permitted to extract elements of this file into
//              other files, subject to this notice and the copyright notice 
//              also being copied into the target files.
//
//              Tech-Know-Ware Ltd shall not be liable for any damages suffered 
//              by anyone or anything due to the use of this file however they 
//				may be caused.
//
//				For more information on UMF, go to:
//
//					http://www.tech-know-ware.com/umf
//
//              Copyright Tech-Know-Ware Ltd, 2001, 2002.
//
//              End of Notice.
//
// Module Tag:  umf - Unified Message Format
// Version:     V.95.6
//
// Changes:
// P. Cordell	2001-06-04	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6


#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif


#ifndef FALSE
#	define FALSE   0
#endif

#ifndef TRUE
#	define TRUE    (!FALSE)
#endif

#define UMF_USE_COMPILED_TAG_INFO

typedef enum 
{ 
	EUMF_OK						= 0,
	EUMF_PARSE_FAILED			= 1000,
	EUMF_ALLOCATION_FAILED,
	EUMF_TOO_MANY_RECURSIONS,
    EUMF_MESSAGE_TOO_SHORT,
    EUMF_PARAMETER_TOO_LONG,
    EUMF_PARAMETER_WRONG_TYPE,
    EUMF_PARAMETER_NOT_FOUND,
	EUMF_PARAMETER_DOES_NOT_EXIST,	// Only used when tagging info is present
	EUMF_OUTPUT_TOO_SMALL
} eumf_error;


typedef unsigned short tumf_unicode;


typedef int tumf_input_location;

typedef int tumf_bool;

typedef struct 
{
	int year;	/* years since 0 */
	int month;	/* months since January - [1,12] */
	int day;	/* day of the month - [1,31] */
} tumf_date;

typedef struct 
{
	int hour;    /* hours since midnight - [0,23] */
	int min;     /* minutes after the hour - [0,59] */
	int sec;     /* seconds after the minute - [0,59] */
} tumf_time;


typedef struct sumf_message_element
{
    char						*p_tag;
    tumf_input_location			tag_location;
	int							tag_number;
    tumf_input_location			value_location;
    struct sumf_message_element	*p_compound;
    struct sumf_message_element	*p_next;
} sumf_message_element;


typedef struct sumf_tag_translation 
{
	char	*text_tag;
	int		numerical_tag;
} sumf_tag_translation;


#define KUMF_MAX_TAG 64


// Users may redefine this function to something more useful
#define fumf_alloc( x, y ) malloc( y )


// These are for the tag info data file
extern const char gumf_is_struct[];
extern const char gumf_is_union[];
extern const char gumf_is_untagged[];

typedef struct sumf_tag_info
{
	const char *p_name;
	struct sumf_tag_info *p_definition;
} sumf_tag_info;


typedef struct sumf_location
{
	sumf_message_element	*p_element;
	sumf_tag_info			*p_tag_info;
} sumf_location;


typedef struct sumf_parse_ctrl
{
    const char				*p_message;
	int						length;
	const char				*p_next;
	const char				*p_message_end;
	eumf_error				error;
    int						allocation_failed;
	sumf_location			root_location;
	sumf_location			base_location;
	int						unget_char;
	int						recursion;
} sumf_parse_ctrl;


//-----------------------------------------------------------------------------
//                   Functions for setting up parsing
//-----------------------------------------------------------------------------


int fumf_is_message_complete( const char *ap_message, int a_length );

void fumf_set_tag_info_root( sumf_parse_ctrl *ap_parse_ctrl, sumf_tag_info *ap_tag_info );

tumf_bool fumf_parse_message( sumf_parse_ctrl *ap_parse_ctrl, 
								const char *ap_message, 
								int a_length );

eumf_error fumf_get_last_error( sumf_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//          Functions for debugging and seeing what is going on
//-----------------------------------------------------------------------------


void fumf_display_parse_info( FILE *ah_fout, sumf_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//          Functions for changing the base location in the message
//-----------------------------------------------------------------------------

tumf_bool fumf_set_location( sumf_parse_ctrl *ap_parse_ctrl,
								const char ac_tagging[], 
								int a_instance,
								sumf_location *ap_old_location );

sumf_location fumf_restore_location( sumf_parse_ctrl *ap_parse_ctrl,
										sumf_location a_base );


//-----------------------------------------------------------------------------
//                       Functions for reading parameters
//-----------------------------------------------------------------------------

tumf_bool fumf_is_present( sumf_parse_ctrl *ap_parse_ctrl, 
									const char ac_tagging[],
									int a_instance );

tumf_bool fumf_read_void( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance );

#define fumf_read_null fumf_read_void	/* For V.95.4 compatibility */

tumf_bool fumf_read_boolean( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									int *ap_boolean );

tumf_bool fumf_read_integer( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									int *ap_integer );

tumf_bool fumf_read_ipv4( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[4] );

tumf_bool fumf_read_ipv6( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[16] );

tumf_bool fumf_read_date( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tumf_date *ap_date );

tumf_bool fumf_read_time( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tumf_time *ap_time );

tumf_bool fumf_read_ascii( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

tumf_bool fumf_read_ascii_dynamic( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_string );

tumf_bool fumf_read_const( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

tumf_bool fumf_read_unicode( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tumf_unicode ac_string[],
									int a_max_length );

tumf_bool fumf_read_unicode_dynamic( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									tumf_unicode **app_string );

tumf_bool fumf_read_byte_array( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length );

tumf_bool fumf_read_byte_array_dynamic( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_data,
									int *ap_length );

tumf_bool fumf_read_embedded( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length );

tumf_bool fumf_read_embedded_dynamic( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char **app_message,
									int *ap_length );

tumf_bool fumf_read_union( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length );

int fumf_read_enumerated_union( sumf_parse_ctrl *ap_parse_ctrl,
									const char ac_tagging[],
									int a_instance, 
									sumf_tag_translation ac_tag_translation[] );


//-----------------------------------------------------------------------------
//                      Functions for clearing up
//-----------------------------------------------------------------------------


void fumf_release( sumf_parse_ctrl *ap_parse_ctrl );


//-----------------------------------------------------------------------------
//                      Functions to support encoding
//-----------------------------------------------------------------------------


int fumf_enc_get_encoded_ascii_length( const char ac_ascii[] );

tumf_bool fumf_enc_generate_encoded_ascii( sumf_parse_ctrl *ap_parse_ctrl,
											char ac_escaped_ascii[], 
											int a_escaped_ascii_max_length,
											const char ac_ascii[] );

int fumf_enc_get_encoded_unicode_length( const tumf_unicode ac_unicode[] );

int fumf_enc_generate_encoded_unicode( sumf_parse_ctrl *ap_parse_ctrl,
										char ac_utf8[], int a_utf8_max_length,
										const tumf_unicode ac_unicode[] );

int fumf_enc_get_encoded_byte_array_length( int a_data_length );

tumf_bool fumf_enc_generate_encoded_byte_array( sumf_parse_ctrl *ap_parse_ctrl,
												char ac_base64[],
												int a_max_length,
												const char *ap_data,
												int a_length );


#ifdef __cplusplus
	}
#endif
#endif


//------------------------------------------------------------------------------
//                 Copyright Tech-Know-Ware Ltd, 2001, 2002.
//------------------------------------------------------------------------------
