//------------------------------------------------------------------------------
//                      Copyright Tech-Know-Ware Ltd, 2001
//------------------------------------------------------------------------------


#ifndef _UMF_LIB_HPP
#define _UMF_LIB_HPP


// File name:   umf-lib.h
//------------------------------------------------------------------------------
// Description: This file contains library functions for reading UMF encoded
//				messages.
//
// Notice:      WARNING: This is an early release of this code, and as such may
//				contain a number of bugs.  If you find a bug, please report it
//				to:
//
//					umf-lib-bugs@tech-know-ware.com
//
//				This file is presented 'as is' and comes with no warrenty of 
//				any kind.  
//
//              This file, or any derivative of it, may be used for any 
//				purpose, including commercial purposes, subject to the use
//				being registered on the Tech-Know-Ware web site at:
//
//					http://www.tech-know-ware.com/umf
//
//              Users are permitted to modify the contents of the file, as 
//				long as this notice and the copyright notice remain.
//
//              Users are permitted to extract elements of this file into
//              other files, subject to this notice and the copyright notice 
//              also being copied into the target files.
//
//              Tech-Know-Ware Ltd shall not be liable for any damages suffered 
//              by anyone or anything due to the use of this file however they 
//				may be caused.
//
//              Copyright Tech-Know-Ware Ltd, 2001, 2002.
//
//              End of Notice.
//
// Module Tag:  umf - Unified Message Format
// Version:     V.95.6
//
// Changes:
// P. Cordell	2001-06-17	Preparation for evaluation
// P. Cordell	2002-03-07	V.95.4 Release
// P. Cordell	2002-04-07	Move to V.95.5
// P. Cordell	2002-04-25	Move to V.95.6


#include "umf-lib.h"

extern "C" tumf_bool fumf_initialise( sumf_parse_ctrl *ap_aprse_ctrl );


class c_umf
{
private:
	sumf_parse_ctrl		o_parse_ctrl;

public:
	c_umf()
		{
		fumf_initialise( &o_parse_ctrl );
		}

	c_umf( char *ap_message, int a_length )  
		{ 
		fumf_parse_message( &o_parse_ctrl, ap_message, a_length ); 
		}


	static int f_is_message_complete( char *ap_message, int a_length )
		{
		return fumf_is_message_complete( ap_message, a_length );
		}

	void f_set_tag_info_root( sumf_tag_info *ap_tag_info )
		{
		fumf_set_tag_info_root( &o_parse_ctrl, ap_tag_info );
		}

	tumf_bool f_parse_message( char *ap_message, int a_length )
		{
		if( fumf_parse_message( &o_parse_ctrl, ap_message, a_length ) )
			return TRUE;

		return FALSE;
		}


	eumf_error f_get_last_error()
		{
		return fumf_get_last_error( &o_parse_ctrl );
		}


	//-----------------------------------------------------------------------------
	//          Functions for debugging and seeing what is going on
	//-----------------------------------------------------------------------------


	void f_display_parse_info( FILE *ah_fout )
		{
		fumf_display_parse_info( ah_fout, &o_parse_ctrl );
		}


	//-----------------------------------------------------------------------------
	//          Functions for changing the base location in the message
	//-----------------------------------------------------------------------------

	tumf_bool f_set_location( char ac_tagging[], int a_instance )
		{
		return fumf_set_location( &o_parse_ctrl, ac_tagging, a_instance, NULL );
		}


	tumf_bool f_set_location( sumf_location *ap_old_location, 
											char ac_tagging[], 
											int a_instance )
		{
		return fumf_set_location( &o_parse_ctrl, ac_tagging, a_instance, ap_old_location );
		}


	sumf_location f_restore_location( sumf_location a_old_location )
		{
		return fumf_restore_location( &o_parse_ctrl, a_old_location );
		}


	//-----------------------------------------------------------------------------
	//                       Functions for reading parameters
	//-----------------------------------------------------------------------------

	tumf_bool f_is_present( char ac_tagging[], int a_instance )
		{
		return fumf_is_present( &o_parse_ctrl, 
									ac_tagging,
									a_instance );
		}


	tumf_bool f_read_void( char ac_tagging[], int a_instance )
		{
		return fumf_read_void( &o_parse_ctrl,
									ac_tagging,
									a_instance );
		}


	tumf_bool f_read_boolean( char ac_tagging[],
									int a_instance, 
									int *ap_boolean )
		{
		return fumf_read_boolean( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ap_boolean );
		}


	tumf_bool f_read_integer( char ac_tagging[],
									int a_instance, 
									int *ap_integer )
		{
		return fumf_read_integer( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ap_integer );
		}


	tumf_bool f_read_ipv4( char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[4] )
		{
		return fumf_read_ipv4( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_addr );
		}


	tumf_bool f_read_ipv6( char ac_tagging[],
									int a_instance, 
									unsigned char ac_addr[16] )
		{
		return fumf_read_ipv6( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_addr );
		}


	tumf_bool f_read_ascii( char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return fumf_read_ascii( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tumf_bool f_read_ascii_dynamic( char ac_tagging[],
									int a_instance, 
									char **app_string )
		{
		return fumf_read_ascii_dynamic( &o_parse_ctrl,
										ac_tagging,
										a_instance, 
										app_string );
		}


	tumf_bool f_read_const( char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return fumf_read_const( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tumf_bool f_read_unicode( char ac_tagging[],
									int a_instance, 
									tumf_unicode ac_string[],
									int a_max_length )
		{
		return fumf_read_unicode( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	tumf_bool f_read_unicode_dynamic( char ac_tagging[],
										int a_instance, 
										tumf_unicode **app_string )
		{
		return fumf_read_unicode_dynamic( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											app_string );
		}


	tumf_bool f_read_byte_array( char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length )
		{
		return fumf_read_byte_array( &o_parse_ctrl,
										ac_tagging,
										a_instance, 
										ac_string,
										a_max_length,
										ap_result_length );
		}


	tumf_bool f_read_byte_array_dynamic( char ac_tagging[],
										int a_instance, 
										char **app_data,
										int *ap_length )
		{
		return fumf_read_byte_array_dynamic( &o_parse_ctrl,
												ac_tagging,
												a_instance, 
												app_data,
												ap_length );
		}


	tumf_bool f_read_embedded( char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length,
									int *ap_result_length )
		{
		return fumf_read_embedded( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length,
									ap_result_length );
		}


	tumf_bool f_read_embedded_dynamic( char ac_tagging[],
										int a_instance, 
										char **app_message,
										int *ap_length )
		{
		return fumf_read_embedded_dynamic( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											app_message,
											ap_length );
		}


	tumf_bool f_read_union( char ac_tagging[],
									int a_instance, 
									char ac_string[],
									int a_max_length )
		{
		return fumf_read_union( &o_parse_ctrl,
									ac_tagging,
									a_instance, 
									ac_string,
									a_max_length );
		}


	int f_read_enumerated_union( char ac_tagging[],
									int a_instance, 
									sumf_tag_translation ac_tag_translation[] )
		{
		return fumf_read_enumerated_union( &o_parse_ctrl,
											ac_tagging,
											a_instance, 
											ac_tag_translation );
		}


	//-----------------------------------------------------------------------------
	//                      Functions for clearing up
	//-----------------------------------------------------------------------------

	
	void f_release()
		{
		fumf_release( &o_parse_ctrl );
		}


	~c_umf()
		{
		fumf_release( &o_parse_ctrl );
		}
};


#endif


//------------------------------------------------------------------------------
//                 Copyright Tech-Know-Ware Ltd, 2001, 2002.
//------------------------------------------------------------------------------
